#!/usr/bin/python
#  Copyright (c) 1999-2020, Virtuozzo International GmbH. All rights reserved.
#
#  This file is part of OpenVZ. OpenVZ is free software; you can redistribute
#  it and/or modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the License,
#  or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
#  02110-1301, USA.
#
#  Our contact details: Virtuozzo International GmbH, Vordergasse 59, 8200
#  Schaffhausen, Switzerland.

import sys
from subprocess32 import call, check_call

try:
    from firewall.core.io.direct import Direct
except:
    # Firewall not installed.
    sys.exit(0)

if len(sys.argv) < 2 or sys.argv[1] not in ["add", "remove"]:
    print "Invalid action. Valid: add|remove."
    sys.exit(1)

FIREWALLD_DIRECT = "/etc/firewalld/direct.xml"
RULE_ARGS = ["-i", "venet0", "-j", "DROP"]
FWD_ARGS  = ["-j", "ACCEPT"]
PORT_ARGS = ["-m", "tcp", "-p", "tcp", "-j", "ACCEPT", "--dport"]
# port list (ipv6 and ipv4) according to:
# http://docs.virtuozzo.com/virtuozzo_7_installation_guide/installing-virtuozzo/configuring-server-ports-for-virtuozzo.html
PORT_LIST = ["22", "80", "389", "4533", "4534", "4648", "4450", "4452", "1433", "64000", "2511", "2510", "2555",
              # VNC ports from dispatcher config
              "5700:6900" ]

class fdirect(object):
    def __init__(self):
        self.d = Direct(FIREWALLD_DIRECT)

    def __enter__(self):
        self.except_pass(self.d.read)
        return self

    def except_pass(self, func, *args):
        try:
            func(*args)
        except:
            pass

    def process_rule(self, *args):
        self.except_pass(getattr(self.d, sys.argv[1] + "_rule"), *args)

    def __exit__(self, exc_type, exc_value, traceback):
        pass

    def __del__(self):
        self.except_pass(self.d.write)

with fdirect() as d:
    for ip in ["ipv4", "ipv6"]:
        for priority in (0, 10):
            # Do not add INPUT_direct venet with zero prio
            if sys.argv[1] == "add" and priority == 0:
                continue
            d.process_rule(ip, "filter", "INPUT_direct", priority, RULE_ARGS)
        d.process_rule(ip, "filter", "FORWARD", 0, FWD_ARGS)
        for port in PORT_LIST:
            d.process_rule(ip, "filter", "INPUT", 0, PORT_ARGS + [port])

    del(d)

try:
    check_call(["/bin/systemctl", "is-active", "firewalld"])
except:
    # Firewall is not running
    sys.exit(0)

dparms = ["/usr/bin/firewall-cmd", "--direct", "--" + sys.argv[1] + "-rule" ]

for ip in ["ipv4", "ipv6"]:
    for priority in (0, 10):
        # Do not add INPUT_direct venet with zero prio
        if sys.argv[1] == "add" and priority == 0:
            continue
        call(dparms + [ip, "filter", "INPUT_direct", "%i" % priority] + RULE_ARGS)
    call(dparms + [ip, "filter", "FORWARD", "0"] + FWD_ARGS)
    for port in PORT_LIST:
        call(dparms + [ip, "filter", "INPUT", "0"] + PORT_ARGS + [port])
